/* Saleh Elmohamed */
/* Threaded applet with double buffering */

// Documentation (What's That For? :-)
//
// Runtime parameters:
// pointsize - font size in points
// style - font style, BOLD, ITALIC, or PLAIN
// font - font name
// text - text to be scrolled
// foregorund - foreground color as three hex numbers, i.e. ff0000 is red.
// background - background color, same format as foreground
// speed - number of units the scrolling text will move each timestep


import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class SalehTickerTape extends java.applet.Applet implements Runnable {
  
  Thread applet = null;
  Image buffer = null, splashimage = null;
  Graphics gBuffer = null;
  URL splashURL, textURL;
  URL homepage;
  MediaTracker tracker;
  Vector message = null;
  Font font;
  Color background, foreground;
  int pointsize, style, step, splashtime, pos1, pos2, pos3, stringwidth;
  int width, height;
  int speed = 50;
  int rerun = 0, count = 0, msgcount = 0;
  int mode;
  boolean suspend = false;
  boolean splash = false; 
  boolean show1 = false;
  boolean show2 = false;
  boolean show3 = false;
  boolean startticker = false;
  boolean textURLfound;
  
  static int LOADING = 0;
  static int SPLASH = 1;
  static int TICKER = 2;
  static int ERROR = 3; 

  public void init() {
    
    String att;

    try {
	homepage = 
       	new URL("http://www.npac.syr.edu/users/saleh/homepage/index3.html");
      } catch (MalformedURLException e) {}
    
    message = new Vector();
    
    width = size().width;
    height = size().height;
    
    tracker = new MediaTracker(this);
    
    try {
      buffer = createImage(width, height);
      gBuffer = buffer.getGraphics();
    } catch (Exception e) {
      gBuffer = null;
    }
    
    startticker = true;
    
    att = getParameter("pointsize");  // font size in points
    pointsize = (att == null)? 27 : Integer.valueOf(att).intValue();
        
    att = getParameter("style");  // font style (BOLD, ITALIC, PLAIN)
    if ( (att != null) && (att.compareTo("BOLD") == 0) ) {
      style = Font.BOLD;
    } else if ( (att != null) && (att.compareTo("ITALIC") == 0) ) {
      style = Font.ITALIC;
    } else {
      style = Font.PLAIN;
    }
      
    att = getParameter("font");  // font
    att = (att == null)? "TimesRoman" : att;
    font = new Font(att, style, pointsize);
       
    att = getParameter("textURL"); // URL for file containing multiple messages
    if ( att != null) {
      try {
        textURL = new URL(att);
      } catch (MalformedURLException e) {
        message.addElement(new String("Cannot load messages - bad URL"));
      }
      if ( textURL != null ) {
        try {
          getText(textURL);
        } catch (IOException e) {
        }
      }
    } else {
        message.addElement(new String("Hi! thanks for the visit  ... the computing field is always in need of new cliches :-) ... inside every large problem is a small problem struggling to get out   ----  the trouble with doing something right the first time is that nobody appreciates how difficult it was    ----   simplicity and order are, if not the principal, then certainly the most important guidlines for human beings in general ... what we see of nature is only a projection of the true reality, as with a shadow cast against the wall of a cave  ----  les mathematiques sont simples, c'est nous qui sommes compliques  ----  it takes a big man to blow through the wrong end of a tuba ... "));   
	}
      
    message.trimToSize();
       
    att = getParameter("foreground");  // text color

    foreground = (att == null)? new Color(Integer.valueOf("000000",16).intValue()) : new Color(Integer.valueOf(att,16).intValue());
  
    att = getParameter("background");  // background color
    background = (att == null)? new Color(Integer.valueOf("87ceeb",16).intValue()) : new Color(Integer.valueOf(att,16).intValue());
    
    att = getParameter("speed"); // scroll speed


    step = (att == null)? 4 : Integer.valueOf(att).intValue();

    att = getParameter("splashtime");  // length of time (s) splash screen is shown
    splashtime = (att == null)? 5000 : 6000*(Integer.valueOf(att).intValue());
 
    att = getParameter("splash");  // Use graphic splash screen (TRUE/FALSE)
    if ( ((att == null)? 1 : att.compareTo("TRUE")) == 0 ) {
      splash = true;
      att = getParameter("splashURL");  // You need a URL for the graphic
      if ( att != null ) {
        try {
          splashURL = new URL(att);
        } 
        catch (MalformedURLException e) {
        }
        if ( splashURL != null ) {    
          splashimage = getImage(splashURL);
          tracker.addImage(splashimage, 0);
          mode = LOADING;
        } else {
          splash = false;
        }
      } else {
        splash = false;
      }
    }
        
  }
  
  public void start() {
    
    if (applet == null) {
      applet = new Thread(this);
      startticker = true;
      if (splash) {
        mode = SPLASH;
      }
      applet.start();
    }
    
  }
  
  public void stop() {
  
    if (applet != null) {
      applet.stop();
      applet = null;
    }
    
  }
  
  public void update(Graphics g) {
   
    paint(g);
    
  }
  
  public void paint(Graphics g) {
    
    if (gBuffer != null) {
      Paint(gBuffer);
      g.drawImage(buffer, 0, 0, this);
    } else {
      Paint(g);
    }
  }
  
  public void run() {
    mode = LOADING;
    repaint();
    if (splash) {
      try {
        tracker.waitForID(0);
      } catch (InterruptedException e) {
        return;
      }
      mode = SPLASH;
    } else {
      mode = TICKER;
    }
    while (applet != null) {
      repaint();
      if (mode == SPLASH) {
        try {
          applet.sleep(splashtime);
        } catch (InterruptedException e) {
          return;
        }
        mode = TICKER;
      }
      try {
        applet.sleep(speed);
      } catch (InterruptedException e) {
        return;
      }
    }
  }
  
  
  public boolean mouseEnter(java.awt.Event evt) 
                                // When the mouse enters the applet
    {
      getAppletContext().showStatus("Hi! Ticker Tape by Saleh");
	    return true;
    }
  
  public boolean mouseExit(java.awt.Event evt)  
                                // When the mouse leaves the applet
    {
      getAppletContext().showStatus("The ticker is running ... cheers :-)");
      return true;
    }

  public boolean mouseDown(java.awt.Event evt, int x,int y) // When a mouse button is pressed 
    {
      getAppletContext().showDocument(homepage);
      return true;
    }

  public void Paint(Graphics g) {
  
    String string;
    int black = (int)(Math.random()* 266) + 228;
    int navy = (int)(Math.random() * 66) + 2;
    int blue = (int)(Math.random() * 1) + 28;
  
    if (mode == LOADING) {
    
      g.setColor(background);
      g.fillRect(0, 0, width, height);
      g.setFont(font);
      FontMetrics fontmet = g.getFontMetrics();
      g.setColor(Color.black);
      g.drawString("LOADING", (width/2) - (fontmet.stringWidth("LOADING")/2), 
        (height/2 + (fontmet.getHeight()/2)));
      return;
      
    } else if (mode == SPLASH) {
    
      g.drawImage(splashimage, 0, 0, this);
      return;
      
    } else if (mode == TICKER) {
    
      if (msgcount == 0) {
        string = String.valueOf(message.firstElement()).toString();
      } else if (msgcount == message.size()) {
        string = String.valueOf(message.firstElement()).toString();
        msgcount = 0;
      } else {
        string = String.valueOf(message.elementAt(msgcount)).toString();
      }
          
      g.setColor(background);
      g.fillRect(0, 0, width, height);
      g.setFont(font);
      FontMetrics fontmet = g.getFontMetrics();
      stringwidth = fontmet.stringWidth(string);
      
      rerun = width + 50 + stringwidth;
      
      if (startticker) {
        pos1 = 0;
        pos2 = width + 20;
        pos3 = rerun;
        startticker = false;
      } else if ( (pos1 - step) <= (0 - rerun) ) {
        if (splash) { 
          mode = SPLASH;
        }
        startticker = true;
        msgcount = msgcount + 1;
        pos1 = 0;
        pos2 = width + 20;
        pos3 = rerun;
      } else {
        pos1 = pos1 - step;
        pos2 = pos2 - step;
        pos3 = pos3 - step;
      }
      show1 = ((pos1 >= 0 - width) && (pos1 <= width))? true : false;
      show2 = ((pos2 >= 0 - stringwidth) && (pos2 <= width))? true: false;
      show3 = ((pos3 >= 0 - width) && (pos3 <= width))? true : false;

      if (splash && show1) {
        g.drawImage(splashimage, pos1, 0, this);
      }
      if (show2) {
/*
	g.setColor(foreground);
*/
	g.setColor(new java.awt.Color(black, navy, blue));
        g.drawString(string, pos2, (height/2 + (fontmet.getHeight())/2));
      }
      if (splash && show3) {
        g.drawImage(splashimage, pos3, 0, this);
      }
      return;
      
    } else { // mode = ERROR
      g.setColor(background);
      g.fillRect(0, 0, width, height);
      g.setFont(font);
      FontMetrics fontmet = g.getFontMetrics();
/*
      g.setColor(Color.red);
*/
      g.setColor(new java.awt.Color(black, navy, blue));
      g.drawString("ERROR", (width/2) - (fontmet.stringWidth("ERROR")/2), 
        (height/2 + (fontmet.getHeight())/2));
      return;
    }
  }
  
  private void getText(URL url) throws java.io.IOException {
  
    InputStream file;
    DataInputStream stream;
    String line;
   
    file = url.openStream();
    stream = new DataInputStream(file);
      
    while((line = stream.readLine()) != null) {
      line = line.trim();
      if (line.length() != 0) {
        message.addElement(new String(line));
      }
        
    }
    return;
    
  } 
}
